#!/usr/bin/env python


#**** BEGIN LICENSE BLOCK *****

# Version: MPL 1.1/GPL 2.0/LGPL 2.1

#   Provides Python evaluation and completions for the PyShell UI.

#

import os
import sys
import time
import traceback
from cStringIO import StringIO

from xpcom import components, ServerException, nsError
from xpcom.server import WrapObject

class pyShell:
    _com_interfaces_ = [components.interfaces.pyIShell]
    _reg_clsid_ = "{4e5c9764-d465-4fef-ae24-8032f257d174}"
    _reg_contractid_ = "@twhiteman.netfirms.com/pyShell;1"
    _reg_desc_ = "Python shell service"

    pyshellGlobals = {
        # Give away some free items...
        "os": os,
        "sys": sys,
        "time": time,
        # And xpcom accessors.
        "components": components,
        "Components": components
    }

    def __init__(self):
        pass

    def _eval_code_and_return_result(self, code):
        return eval(code, self.pyshellGlobals, self.pyshellGlobals)

    # This little exec snippet comes from the python mailing list, see:
    # http://mail.python.org/pipermail/python-list/2005-June/328628.html
    def _exec_code_and_get_output(self, code):
        old_stdout = sys.stdout
        sys.stdout = StringIO()
        try:
            exec code in self.pyshellGlobals, self.pyshellGlobals
            return sys.stdout.getvalue()
        finally:
            sys.stdout = old_stdout
 
    def evalPythonString(self, code):
        # Ensure the code ends with an empty newline
        code += '\n\n'
        try:
            try:
                result = self._eval_code_and_return_result(code)
                try:
                    # See if the result can be turned into an xpcom object
                    return WrapObject(result, components.interfaces.nsIVariant)
                except ValueError:
                    # else, we'll just return a string representation
                    return repr(result)
            except SyntaxError:
                return self._exec_code_and_get_output(code)
        except Exception, e:
            # Format the exception, removing the exec/eval sections.
            exc_tb = traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback)
            return "".join(exc_tb[:1] + exc_tb[3:])

    def getCompletionsForName(self, objname, prefix):
        #print "getCompletionsForName:: obname: %r, prefix: %r" % (objname, prefix, )
        # Global scope.
        if not objname:
            cplns = self.pyshellGlobals.keys()
        # Hack for nice xpcom completions.
        elif objname.lower() == "components.interfaces":
            cplns = components.interfaces.keys()
        elif objname.lower() == "components.classes":
            cplns = components.classes.keys()
        # Object scope.
        else:
            foundObject = None
            names = objname.split(".")
            foundObject = self.pyshellGlobals[names[0]]
            for name in names[1:]:
                foundObject = getattr(foundObject, name)
            # Got the object, now return the matches
            cplns = dir(foundObject)

        if prefix:
            cplns = [x for x in cplns if x.startswith(prefix)]
        return cplns

